#include <windows.h>

#include "ObjectViewSwitcherWindow.h"
#include "ObjectViewSwitcher.h"

ObjectViewSwitcherWindow::ObjectViewSwitcherWindow(ObjectViewSwitcher *plugin, MQWindowBase& parent)
	: MQWindow(parent), m_pPlugin(plugin), m_shift(false)
{
	SetTitle(L"Switcher");
	SetCloseButton(false);
	SetCanResize(false);
	
	m_parentframe = CreateVerticalFrame(this);
	createButtonList(m_parentframe);

	MQButton *button_add = CreateButton(this, L"Add");
	button_add->AddClickEvent(this, &ObjectViewSwitcherWindow::f_add);

	AddHideEvent(this, &ObjectViewSwitcherWindow::OnHide);
	InitializeCriticalSection(&m_critical);
}


void ObjectViewSwitcherWindow::createButtonList(MQWidgetBase *parent)
{
	m_buttonframe = CreateHorizontalFrame(parent);
	m_buttonframe->SetMatrixColumn(2);
	m_buttonframe->SetInSpace(0);
	m_buttonframe->SetOutSpace(0);
	for(int i = 0; i < m_pPlugin->stragesize(); i++) {
		MQButton *m_button_set = CreateButton(m_buttonframe);
		m_button_set->SetText(L"S");
		m_button_set->AddLeftDownEvent(this, &ObjectViewSwitcherWindow::f_set);
		m_buttons.push_back(m_button_set);
		m_button_set->SetOutSpace(0);
		m_button_set->SetInSpace(0);
		if(m_pPlugin->hasdata(i)) {
			m_button_set->SetDown(true);
		}

		MQButton *m_button_restore = CreateButton(m_buttonframe);
		m_button_restore->SetText(L"R");
		m_button_restore->AddLeftDownEvent(this, &ObjectViewSwitcherWindow::f_restore);
		m_button_restore->SetOutSpace(0);
		m_button_restore->SetInSpace(0);
		m_buttons.push_back(m_button_restore);
	}
}

void ObjectViewSwitcherWindow::destroyButtonList()
{
	m_parentframe->RemoveChild(m_buttonframe);
	delete m_buttonframe;
	m_buttons.clear();
}

ObjectViewSwitcherWindow::~ObjectViewSwitcherWindow()
{
	DeleteCriticalSection(&m_critical);
	//obj_.deleteThis();
}

void ObjectViewSwitcherWindow::clearbutton() {
	for(auto button : m_buttons) {
		button->SetDown(false);
	}
}

void ObjectViewSwitcherWindow::setbutton(int index, bool flag) {
	if(m_buttons.size() >= (index + 1) * 2) {
		m_buttons[index * 2]->SetDown(flag);
	}
}

BOOL ObjectViewSwitcherWindow::f_set(MQWidgetBase *sender, MQDocument doc, MQWidgetMouseParam &param)
{
	auto findit = std::find(m_buttons.begin(), m_buttons.end(), sender);
	if(findit != m_buttons.end()) {
		int index = (findit - m_buttons.begin()) / 2;
		if(param.Ctrl) {
			// Îݓo^
			if((*findit)->GetDown()) {
				//ǉo^
				MQDialog::DIALOG_RESULT result = MQDialog::MessageOkCancelBox(*this, L"Append?", L"Switcher");
				if(result != MQDialog::DIALOG_OK) {
					return FALSE;
				}
			}
			(*findit)->SetDown(true);
			return m_pPlugin->set_list(doc, index, true);
		}
		else if(param.Shift) {
			// 폜
			if((*findit)->GetDown()) {
				MQDialog::DIALOG_RESULT result = MQDialog::MessageOkCancelBox(*this, L"Clear?", L"Switcher");
				if(result != MQDialog::DIALOG_OK) {
					return FALSE;
				}
			}
			(*findit)->SetDown(false);
			return m_pPlugin->clear_list(doc, index);
		}
		else {
			// o^
			if((*findit)->GetDown()) {
				MQDialog::DIALOG_RESULT result = MQDialog::MessageOkCancelBox(*this, L"Overwrite?", L"Switcher");
				if(result != MQDialog::DIALOG_OK) {
					return FALSE;
				}
			}
			(*findit)->SetDown(true);
			return m_pPlugin->set_list(doc, index);
		}
	}
	return FALSE;
}

BOOL ObjectViewSwitcherWindow::f_restore(MQWidgetBase *sender, MQDocument doc, MQWidgetMouseParam &param)
{
	auto findit = std::find(m_buttons.begin(), m_buttons.end(), sender);
	if(findit != m_buttons.end()) {
		int index = (findit - m_buttons.begin()) / 2;
		return m_pPlugin->restore_list(doc, index);
	}
	return FALSE;
}

BOOL ObjectViewSwitcherWindow::f_add(MQWidgetBase *sender, MQDocument doc)
{
	m_pPlugin->addSlot();
	resizeButtonList();
	return FALSE;
}

void ObjectViewSwitcherWindow::resizeButtonList()
{
	EnterCriticalSection(&m_critical);
	destroyButtonList();
	createButtonList(m_parentframe);
	resize();
	LeaveCriticalSection(&m_critical);
}

BOOL ObjectViewSwitcherWindow::OnHide(MQWidgetBase *sender, MQDocument doc)
{
	if(m_pPlugin->m_bActivate){
		m_pPlugin->m_bActivate = false;
		m_pPlugin->WindowClose();
	}
	return FALSE;
}

void ObjectViewSwitcherWindow::resize()
{
	POINT size = GetJustSize();
	SetWidth(size.x);
	SetHeight(size.y);
}
